/* Checks if the algorithm converges regardless of where it starts */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cassert>

#include <dune/common/shared_ptr.hh>

#include <dune/tectonic/ellipticenergy.hh>

#include "test-gradient-method-nicefunction.hh"
#include "test-gradient-method-helper.hh"

int main() {
  int const dim = 3;
  typedef Dune::EllipticEnergy<dim> Functional;
  typedef Functional::SmallMatrix SmallMatrix;
  typedef Functional::SmallVector SmallVector;

  SmallMatrix A(0);
  A[0][0] = 3;
  A[0][1] = A[1][0] = 1.5;
  A[1][1] = 4;
  A[1][2] = A[2][1] = 1.5;
  A[2][2] = 5;
  SmallVector b;
  b[0] = 1;
  b[1] = 2;
  b[2] = 3;

  auto const f = Dune::make_shared<Dune::SampleFunction<2> const>();
  auto const phi = Dune::make_shared<Functional::NonlinearityType const>(f);
  Functional J(A, b, phi);

  SmallVector start = b;
  start *= 17;

  double const ret1 = functionTester(J, start, 9);

  // Something random
  start[0] = 279;
  start[1] = -96;
  start[2] = -15;

  double const ret2 = functionTester(J, start, 15);
  assert(std::abs(ret1 - ret2) < 1e-5);

  start[0] = 0;
  start[1] = 0;
  start[2] = 0;

  double const ret3 = functionTester(J, start, 5);
  assert(std::abs(ret1 - ret3) < 1e-5);
}
